==> pathsim-0.1.0.tar.gz: pathsim-0.1.0/pathsim/solvers/ssprk33.py <==
########################################################################################
##
##               EXPLICIT STRONG STABILITY PRESERVING RUNGE-KUTTA INTEGRATOR
##                                (solvers/ssprk33.py)
##
##                                 Milan Rother 2024
##
########################################################################################

# IMPORTS ==============================================================================

from ._solver import ExplicitSolver


# SOLVERS ==============================================================================

class SSPRK33(ExplicitSolver):
    """
    Strong Stability Preserving (SSP) 3-rd order 
    three stage (3,3) Runge-Kutta method
    
    This integrator is more accurate and stable then SSPRK22 but 
    also 50% more expensive due to 3 instead of 2 stages. 
    Originally designed for hyperbolic PDEs, this is also a great 
    choice if accuracy and stability and still good speed are important.
    """

    def __init__(self, initial_value=0, func=lambda x, u, t: u, jac=None, tolerance_lte=1e-6):
        super().__init__(initial_value, func, jac, tolerance_lte)

        #counter for runge kutta stages
        self.stage = 0

        #slope coefficients for stages
        self.Ks = {}

        #intermediate evaluation times
        self.eval_stages = [0.0, 1.0, 0.5]

        #butcher table
        self.BT = {0:[1.0],
                   1:[1/4, 1/4],
                   2:[1/6, 1/6, 2/3]}


    def step(self, u, t, dt):
        """
        performs the (explicit) timestep for (t+dt) 
        based on the state and input at (t)
        """

        #buffer intermediate slope
        self.Ks[self.stage] = self.func(self.x, u, t)
        
        #update state at stage
        slope = 0.0
        for i, b in enumerate(self.BT[self.stage]):
            slope += self.Ks[i] * b
        self.x = dt * slope + self.x_0

        #wrap around stage counter
        self.stage = (self.stage + 1) % 3

        #no error estimate available
        return True, 0.0, 1.0

==> pathsim-0.4.7.tar.gz: pathsim-0.4.7/pathsim/solvers/ssprk33.py <==
########################################################################################
##
##               EXPLICIT STRONG STABILITY PRESERVING RUNGE-KUTTA INTEGRATOR
##                                (solvers/ssprk33.py)
##
##                                 Milan Rother 2024
##
########################################################################################

# IMPORTS ==============================================================================

from ._rungekutta import ExplicitRungeKutta


# SOLVERS ==============================================================================

class SSPRK33(ExplicitRungeKutta):
    """
    Strong Stability Preserving (SSP) 3-rd order 
    three stage (3,3) Runge-Kutta method
    
    This integrator is more accurate and stable then SSPRK22 but 
    also 50% more expensive due to 3 instead of 2 stages. 
    Originally designed for hyperbolic PDEs, this is also a great 
    choice if accuracy and stability and still good speed are important.
    """

    def __init__(self, *solver_args, **solver_kwargs):
        super().__init__(*solver_args, **solver_kwargs)

        #number of stages in RK scheme
        self.s = 3

        #order of scheme
        self.n = 3

        #intermediate evaluation times
        self.eval_stages = [0.0, 1.0, 0.5]

        #butcher table
        self.BT = {0:[1.0],
                   1:[1/4, 1/4],
                   2:[1/6, 1/6, 2/3]}

    def interpolate(self, r, dt):
        """
        b₁(θ) = θ(1-θ)²/2
        b₂(θ) = θ²(3-2θ)/2
        b₃(θ) = θ³

        y(t₀ + θh) = y₀ + h(b₁(θ) * k₁ + b₂(θ) * k₂ + b₃(θ) * k₃)
        """
        k1, k2, k3 = self.K[0], self.K[1], self.K[2]
        b1, b2, b3 = r*(2-r)**2/2, r**2*(3-2*r)/2, r**3
        return self.x_0 + dt*(b1 * k1 + b2 * k2 + b3 * k3)
